# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.debugging.experimental namespace.
"""

import sys as _sys

from tensorflow.python.debug.lib.dumping_callback import disable_dump_debug_info
from tensorflow.python.debug.lib.dumping_callback import enable_dump_debug_info