# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================

"""Contains the core layers: Dense, Dropout.

Also contains their functional aliases.
"""
from tensorflow.python.keras.legacy_tf_layers import core


Dense = core.Dense
dense = core.dense
Dropout = core.Dropout
dropout = core.dropout
Flatten = core.Flatten
flatten = core.flatten

# Aliases

FullyConnected = Dense
fully_connected = dense
