syntax = "proto3";

package tensorflow.dummy;

import public "tensorflow/tsl/protobuf/bfc_memory_map.proto";

option go_package = "github.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto";
