# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.__internal__.eager_context namespace.
"""

import sys as _sys

from tensorflow.python.eager.context import eager_mode
from tensorflow.python.eager.context import get_config
from tensorflow.python.eager.context import get_device_name
from tensorflow.python.eager.context import get_executor
from tensorflow.python.eager.context import set_soft_device_placement