# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/util/event.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import summary_pb2 as tensorflow_dot_core_dot_framework_dot_summary__pb2
try:
  tensorflow_dot_tsl_dot_protobuf_dot_histogram__pb2 = tensorflow_dot_core_dot_framework_dot_summary__pb2.tensorflow_dot_tsl_dot_protobuf_dot_histogram__pb2
except AttributeError:
  tensorflow_dot_tsl_dot_protobuf_dot_histogram__pb2 = tensorflow_dot_core_dot_framework_dot_summary__pb2.tensorflow.tsl.protobuf.histogram_pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tensorflow/core/util/event.proto\x12\ntensorflow\x1a\'tensorflow/core/framework/summary.proto\"\xf4\x02\n\x05\x45vent\x12\x11\n\twall_time\x18\x01 \x01(\x01\x12\x0c\n\x04step\x18\x02 \x01(\x03\x12\x16\n\x0c\x66ile_version\x18\x03 \x01(\tH\x00\x12\x13\n\tgraph_def\x18\x04 \x01(\x0cH\x00\x12&\n\x07summary\x18\x05 \x01(\x0b\x32\x13.tensorflow.SummaryH\x00\x12\x31\n\x0blog_message\x18\x06 \x01(\x0b\x32\x16.tensorflow.LogMessageB\x02\x18\x01H\x00\x12-\n\x0bsession_log\x18\x07 \x01(\x0b\x32\x16.tensorflow.SessionLogH\x00\x12<\n\x13tagged_run_metadata\x18\x08 \x01(\x0b\x32\x1d.tensorflow.TaggedRunMetadataH\x00\x12\x18\n\x0emeta_graph_def\x18\t \x01(\x0cH\x00\x12\x33\n\x0fsource_metadata\x18\n \x01(\x0b\x32\x1a.tensorflow.SourceMetadataB\x06\n\x04what\" \n\x0eSourceMetadata\x12\x0e\n\x06writer\x18\x01 \x01(\t\"\xa1\x01\n\nLogMessage\x12+\n\x05level\x18\x01 \x01(\x0e\x32\x1c.tensorflow.LogMessage.Level\x12\x0f\n\x07message\x18\x02 \x01(\t\"Q\n\x05Level\x12\x0b\n\x07UNKNOWN\x10\x00\x12\r\n\tDEBUGGING\x10\n\x12\x08\n\x04INFO\x10\x14\x12\x08\n\x04WARN\x10\x1e\x12\t\n\x05\x45RROR\x10(\x12\t\n\x05\x46\x41TAL\x10\x32\x1a\x02\x18\x01:\x02\x18\x01\"\xb6\x01\n\nSessionLog\x12\x34\n\x06status\x18\x01 \x01(\x0e\x32$.tensorflow.SessionLog.SessionStatus\x12\x17\n\x0f\x63heckpoint_path\x18\x02 \x01(\t\x12\x0b\n\x03msg\x18\x03 \x01(\t\"L\n\rSessionStatus\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\t\n\x05START\x10\x01\x12\x08\n\x04STOP\x10\x02\x12\x0e\n\nCHECKPOINT\x10\x03\"6\n\x11TaggedRunMetadata\x12\x0b\n\x03tag\x18\x01 \x01(\t\x12\x14\n\x0crun_metadata\x18\x02 \x01(\x0c\"$\n\x0eWatchdogConfig\x12\x12\n\ntimeout_ms\x18\x01 \x01(\x03\"&\n\x11RequestedExitCode\x12\x11\n\texit_code\x18\x01 \x01(\x05\"\xb6\x01\n\x16WorkerHeartbeatRequest\x12\x35\n\rshutdown_mode\x18\x01 \x01(\x0e\x32\x1e.tensorflow.WorkerShutdownMode\x12\x33\n\x0fwatchdog_config\x18\x02 \x01(\x0b\x32\x1a.tensorflow.WatchdogConfig\x12\x30\n\texit_code\x18\x03 \x01(\x0b\x32\x1d.tensorflow.RequestedExitCode\"\x83\x01\n\x17WorkerHeartbeatResponse\x12/\n\rhealth_status\x18\x01 \x01(\x0e\x32\x18.tensorflow.WorkerHealth\x12%\n\nworker_log\x18\x02 \x03(\x0b\x32\x11.tensorflow.Event\x12\x10\n\x08hostname\x18\x03 \x01(\t*[\n\x0cWorkerHealth\x12\x06\n\x02OK\x10\x00\x12\x1c\n\x18RECEIVED_SHUTDOWN_SIGNAL\x10\x01\x12\x12\n\x0eINTERNAL_ERROR\x10\x02\x12\x11\n\rSHUTTING_DOWN\x10\x03*k\n\x12WorkerShutdownMode\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x12\n\x0eNOT_CONFIGURED\x10\x01\x12\x18\n\x14WAIT_FOR_COORDINATOR\x10\x02\x12\x1a\n\x16SHUTDOWN_AFTER_TIMEOUT\x10\x03\x42p\n\x13org.tensorflow.utilB\x0b\x45ventProtosP\x01ZGgithub.com/tensorflow/tensorflow/tensorflow/go/core/util/event_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.util.event_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023org.tensorflow.utilB\013EventProtosP\001ZGgithub.com/tensorflow/tensorflow/tensorflow/go/core/util/event_go_proto\370\001\001'
  _EVENT.fields_by_name['log_message']._options = None
  _EVENT.fields_by_name['log_message']._serialized_options = b'\030\001'
  _LOGMESSAGE_LEVEL._options = None
  _LOGMESSAGE_LEVEL._serialized_options = b'\030\001'
  _LOGMESSAGE._options = None
  _LOGMESSAGE._serialized_options = b'\030\001'
  _WORKERHEALTH._serialized_start=1300
  _WORKERHEALTH._serialized_end=1391
  _WORKERSHUTDOWNMODE._serialized_start=1393
  _WORKERSHUTDOWNMODE._serialized_end=1500
  _EVENT._serialized_start=90
  _EVENT._serialized_end=462
  _SOURCEMETADATA._serialized_start=464
  _SOURCEMETADATA._serialized_end=496
  _LOGMESSAGE._serialized_start=499
  _LOGMESSAGE._serialized_end=660
  _LOGMESSAGE_LEVEL._serialized_start=575
  _LOGMESSAGE_LEVEL._serialized_end=656
  _SESSIONLOG._serialized_start=663
  _SESSIONLOG._serialized_end=845
  _SESSIONLOG_SESSIONSTATUS._serialized_start=769
  _SESSIONLOG_SESSIONSTATUS._serialized_end=845
  _TAGGEDRUNMETADATA._serialized_start=847
  _TAGGEDRUNMETADATA._serialized_end=901
  _WATCHDOGCONFIG._serialized_start=903
  _WATCHDOGCONFIG._serialized_end=939
  _REQUESTEDEXITCODE._serialized_start=941
  _REQUESTEDEXITCODE._serialized_end=979
  _WORKERHEARTBEATREQUEST._serialized_start=982
  _WORKERHEARTBEATREQUEST._serialized_end=1164
  _WORKERHEARTBEATRESPONSE._serialized_start=1167
  _WORKERHEARTBEATRESPONSE._serialized_end=1298
# @@protoc_insertion_point(module_scope)
