# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/framework/model.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tensorflow/core/framework/model.proto\x12\x15tensorflow.data.model\"\xf4\x07\n\nModelProto\x12;\n\x05nodes\x18\x01 \x03(\x0b\x32,.tensorflow.data.model.ModelProto.NodesEntry\x12\x0e\n\x06output\x18\x02 \x01(\x03\x12\x12\n\nid_counter\x18\x03 \x01(\x03\x12Q\n\x13optimization_params\x18\x05 \x01(\x0b\x32\x34.tensorflow.data.model.ModelProto.OptimizationParams\x1a\xbf\x04\n\x04Node\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x10\n\x08\x61utotune\x18\x03 \x01(\x08\x12\x16\n\x0e\x62uffered_bytes\x18\x04 \x01(\x03\x12\x19\n\x11\x62uffered_elements\x18\x05 \x01(\x03\x12\x16\n\x0e\x62ytes_consumed\x18\x06 \x01(\x03\x12\x16\n\x0e\x62ytes_produced\x18\x07 \x01(\x03\x12\x14\n\x0cnum_elements\x18\x08 \x01(\x03\x12\x17\n\x0fprocessing_time\x18\t \x01(\x03\x12\x16\n\x0erecord_metrics\x18\n \x01(\x08\x12\x44\n\nparameters\x18\x0b \x03(\x0b\x32\x30.tensorflow.data.model.ModelProto.Node.Parameter\x12!\n\x19input_processing_time_sum\x18\x0c \x01(\x01\x12#\n\x1binput_processing_time_count\x18\r \x01(\x03\x12\x0e\n\x06inputs\x18\x0e \x03(\x03\x12\x34\n\nnode_class\x18\x0f \x01(\x0e\x32 .tensorflow.data.model.NodeClass\x12\r\n\x05ratio\x18\x10 \x01(\x01\x12\x14\n\x0cmemory_ratio\x18\x11 \x01(\x01\x1ah\n\tParameter\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x13\n\x0bstate_value\x18\x03 \x01(\x01\x12\x0b\n\x03min\x18\x04 \x01(\x01\x12\x0b\n\x03max\x18\x05 \x01(\x01\x12\x0f\n\x07tunable\x18\x06 \x01(\x08\x1aT\n\nNodesEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.tensorflow.data.model.ModelProto.Node:\x02\x38\x01\x1a\x93\x01\n\x12OptimizationParams\x12;\n\talgorithm\x18\x01 \x01(\x0e\x32(.tensorflow.data.model.AutotuneAlgorithm\x12\x12\n\ncpu_budget\x18\x02 \x01(\x03\x12\x12\n\nram_budget\x18\x03 \x01(\x03\x12\x18\n\x10model_input_time\x18\x04 \x01(\x01J\x04\x08\x04\x10\x05*\x9c\x01\n\tNodeClass\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x13\n\x0fINTERLEAVE_MANY\x10\x01\x12\x19\n\x15\x41SYNC_INTERLEAVE_MANY\x10\x02\x12\x0f\n\x0bKNOWN_RATIO\x10\x03\x12\x15\n\x11\x41SYNC_KNOWN_RATIO\x10\x04\x12\x11\n\rUNKNOWN_RATIO\x10\x05\x12\x17\n\x13\x41SYNC_UNKNOWN_RATIO\x10\x06*l\n\x11\x41utotuneAlgorithm\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0e\n\nHILL_CLIMB\x10\x01\x12\x14\n\x10GRADIENT_DESCENT\x10\x02\x12\x13\n\x0fMAX_PARALLELISM\x10\x03\x12\x0f\n\x0bSTAGE_BASED\x10\x04\x42QZLgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/model_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.framework.model_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZLgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/model_go_proto\370\001\001'
  _MODELPROTO_NODESENTRY._options = None
  _MODELPROTO_NODESENTRY._serialized_options = b'8\001'
  _NODECLASS._serialized_start=1080
  _NODECLASS._serialized_end=1236
  _AUTOTUNEALGORITHM._serialized_start=1238
  _AUTOTUNEALGORITHM._serialized_end=1346
  _MODELPROTO._serialized_start=65
  _MODELPROTO._serialized_end=1077
  _MODELPROTO_NODE._serialized_start=260
  _MODELPROTO_NODE._serialized_end=835
  _MODELPROTO_NODE_PARAMETER._serialized_start=731
  _MODELPROTO_NODE_PARAMETER._serialized_end=835
  _MODELPROTO_NODESENTRY._serialized_start=837
  _MODELPROTO_NODESENTRY._serialized_end=921
  _MODELPROTO_OPTIMIZATIONPARAMS._serialized_start=924
  _MODELPROTO_OPTIMIZATIONPARAMS._serialized_end=1071
# @@protoc_insertion_point(module_scope)
